//Written by Minghao Wei for Technologic System
//June 2nd, 2016
//The way to use this API is similar to use Object Oriented programming
//First you need to use adcSetup to setup the pointer to the adc
//after that, you just need that pointer to access all the functions
//Please see the example test code for more detail of how to use this API
#include <stdio.h>
#include <stdint.h>
#include <assert.h>
#include <unistd.h>
#include <errno.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <sys/stat.h>

#include "adc16lib.h"

//adc16_base is the base DIO address of your adc16.
//PC104BASE is the base pc104 address of you SBC
volatile uint16_t* adcSetup(uint16_t adc16_base, uint32_t PC104BASE){
        int pc104id =0;
        int mem = 0;
        volatile uint16_t *adc16_io;
        volatile uint16_t *pc104_base = 0;
        //setup device pointer
        mem = open("/dev/mem", O_RDWR|O_SYNC);
        pc104_base = mmap(0, getpagesize(), PROT_READ | PROT_WRITE, MAP_SHARED,mem, PC104BASE);
        adc16_io = pc104_base + (adc16_base/2);
        //printf("returning pointer to address 0x%x\n", adc16_io);
        return adc16_io;
}

//adc16_io is the pointer created in adcSetup
//set register 2, please see manual for information about this register
//pair number should be between 0-7
//start the state machine with selected pair number
void adcStart(volatile uint16_t* adc16_io, int pairNum){
	//state machine state with bit 0x161
	//setting pairNum to bit 1-4
        uint16_t startOption = 0x161 | (pairNum << 1);
        //printf("start option 0x%x\n", startOption);
        adc16_io[2/2] = startOption;
}

//adc16_io is the pointer created in adcSetup
//set register 2
//Please see adc16 manual for more information about this register
void adcStop(volatile uint16_t* adc16_io){
	//set bit 0(SYSCOM) to 0 will stop the state machine
        adc16_io[2/2] = 0x160;//stop state machine
        usleep(100);
}

//read sample from thie FIFO, read register 0xA
uint16_t readFIFO(volatile uint16_t* adc16_io){
        return adc16_io[0xa/2];//ADCFIFO
}

//Set the clock speed, bigger number means slower
//speed = sample delay, ADCDLY is 32-bit number, using 2 register
void setClockSpeed(volatile uint16_t* adc16_io, uint32_t speed){
        uint16_t lsb = 0;
        uint16_t msb = 0;
        lsb = (uint16_t)((uint32_t)(speed & 0x0000ffff));
        msb = (uint16_t)((uint32_t)(speed & 0xffff0000) >> 16);
        adc16_io[4/2] = msb;
        adc16_io[6/2] = lsb;
//Please see manual about register 4, 6.
}

//Clean the FIFO, stop the state machine before using this function
void cleanFIFO(volatile uint16_t* adc16_io){
        int temp;
        unsigned int sample_count = adc16_io[8/2] >> 6;
        printf("FIFO count : %d \n", sample_count);//debugging purpose
        while((adc16_io[0x8/2] >> 6) != 0x0){//FIFO count
                temp = adc16_io[0xa/2];//read FIFO
                fflush(stdout);
                usleep(2);//some delay is necessary 
                //printf("Count: %d, ", (adc16_io[0x8/2] >> 6));
//printf("Channel=%d, Value=0x%4x\n",((adc16_io[0x8/2]&0x1f)>>1), adc16_io[0xa/2]);
        }
        //Checking the FIFO again before leaving the function
        if ((adc16_io[0x8/2] >> 6) != 0x0){
                printf("Error cleaning FIFO, %d did not clean\n", (adc16_io[0x8/2] >> 6));
        }
}

//set the interrupt level, default is half full 256
void setInterrupt(volatile uint16_t* adc16_io, uint16_t level){
        uint16_t interrupt = (level << 6) | 0x1;
        //printf("interrupt 0x%x\n", interrupt);
        adc16_io[8/2] = interrupt;
}

//getting the counter 0-3 's value
//input must be between 0-3
uint16_t getCount(volatile uint16_t* adc16_io, int count){
        if(count == 0){return adc16_io[10/2];}//counter 0
        else if(count == 1){return adc16_io[12/2];}//counter 1
        else if(count == 2){return adc16_io[14/2];}//counter 2
        else if(count == 3){return adc16_io[16/2];}//counter 3
        else{
                printf("ERROR input for getCount\n");
                return 0;
        }
}

//Set DAC output, level fff-000, fff = 5v
//MAX value 5V, if you need MAX value to be 2.5V
//change value in if-statement to 0x1000, 0x5000, 0x9000, 0xd0000
//Please see manual about information of register 0xE
int setDAC(volatile uint16_t* adc16, int dacNum, uint16_t level){
        uint16_t dacLevel = 0;
        if(dacNum == 0){//DAC A, bit 14-15 = 00
                dacLevel = 0x3000;
        }else if(dacNum == 1){//DAC B, bit 14-15 = 01
                dacLevel = 0x7000;
        }else if(dacNum == 2){//DAC C, bit 14-15 = 10
                dacLevel = 0xb000;
        }else if(dacNum == 3){//DAC D, bit 14-15 = 11
                dacLevel = 0xf000;
        }else{
                printf("ERROR input for setDAC\n");
                return -1;
        }
        adc16[0xe/2] = (dacLevel | (level & 0x0fff));
        //printf("dac setting to 0x%x\n", adc16[0xe/2]);
        usleep(5);//some delay is necessary
        return 1;
}

//print board information to the console
//output: board ID, PLD revision, jumperStatus
//ID should be 0x3e for TS-ADC16
void getBoardInfor(volatile uint16_t* adc16_io){
        uint8_t id = adc16_io[0] & 0x00ff;//board ID
        uint8_t PLDRevision = (adc16_io[0] >> 8) & 0x0f;
        uint8_t jumperStatus = adc16_io[0] >> 12;
        printf("ID: 0x%x\nPLD Revision: 0x%x\nJumper Status: 0x%x\n", id, PLDRevision, jumperStatus);
}

//return the current FIFO count
//please see the ADC16 manual for more information about register 0x8
uint16_t getFIFOcount(volatile uint16_t* adc16_io){
        return adc16_io[0x8/2] >> 6;
}

//return the current channel in the head of FIFO
//please see the ADC16 manual for more information about register 0x8
uint8_t getHeadChannel(volatile uint16_t* adc16_io){
        return (adc16_io[0x8/2] >> 1) & 0x1f;
}

//set the DIGIO output bit, output can be either 0 or 1
//See the ADC16 manual for more information about register 0x18
void setDIGIO(volatile uint16_t* adc16_io, int output){
        if(output == 0){
                adc16_io[0x18/2] = 0x10;//writting to bit 5
        }else if(output == 1){
                adc16_io[0x18/2] = 0x00;
        }
}

//return the DIGIO input with respect to the user input
//0-input 0 data, 1-input 1 data......input can be between 0-3
//see the ADC16 manual for more information about register 0x18
uint8_t getDIGIO(volatile uint16_t* adc16_io, int input){
        if(input == 0){//return bit 0
                return adc16_io[0x18/2] & 0x1;
        }else if (input == 1){//return bit 1
                return (adc16_io[0x18/2] >> 1) & 0x1;
        }else if(input == 2){//return bit 2
                return (adc16_io[0x18/2] >> 2) & 0x1;
        }else if(input == 3){//return bit 3
                return (adc16_io[0x18/2] >> 3) & 0x1;
        }else{
                return 0;//Error input
        }
}
   